\encoding{UTF8}
\name{qgraph}
\alias{qgraph}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{qgraph}
\description{
This is the main function of qgraph. It can create several types of networks (undirected/directed and unweighted/weighted), but is typically used to visualize a correlation matrix.}
\usage{
qgraph( adj, ... )
}
\arguments{
	\item{adj}{Can be either an adjacency matrix (recommended) or an edgelist. An adjacency matrix is a square matrix in which each row and column represents a node and the element between each row and column the edge between those two nodes. Typically this is a correlation matrix, but any type of adjacency matrix can be used. If this matrix only contains binary values, an unweighted graph is created. If this matrix is not symmetrical, a directional graph is created. The edgelist is a matrix with a row for each edge containing the origin in the first column the destination in the second column and the weight (optional) in the third column. Origin and destination node must be coded by a number between 1 and the total number of nodes. Can also be an object of class "sem" (sem), "mod" (sem), "lavaan" (lavaan), "principal" (psych), "loadings" (stats), "factanal" (stats), "graphNEL" (Rgraphviz) or "pcAlgo" (pcalg)}
	\item{...}{Any additional arguments described below. Also a list with class "qgraph" can be added that contains any of these arguments (this is returned by the function)}
}	
\section{Important additional arguments}{	
\describe{
\item{layout}{This argument controls the layout of the graph. "circle" gives a circular layout, "groups" gives a layout in which each group is put in a circle and "spring" gives a force embedded layout. It also can be a matrix with a row for each node and x and y coordinates in the first and second column respectively. Defaults to "circle" in weighted graphs without a groups list, "groups" in weighted graphs with a groups list, and "spring" in unweighted graphs.}
\item{groups}{A list object with the measurement model. Each element represents one factor, and must contain the numbers of the nodes in that factor (same order as in the adjacency matrix). This can be used to group the nodes and give them colors.}
\item{minimum}{Edges with absolute weights under this value are omitted. Defaults to 0 for graphs with less than 50 nodes or 0.1 for larger graphs.}
\item{maximum}{qgraph regards the highest of the maximum or highest absolute edge weight as the highest weight to scale the edge widths too. To compare several graphs, set this argument to a higher value than any edge weight in the graphs (typically 1 for correlations).}
\item{cut}{In weighted graphs, this argument can be used to make edges with absolute weights over this value of the strongest color intensity and become wider the stronger they are, and edges with absolute weights under this value of the smallest width and become vaguer the smaller the weight. If this is set to NULL, no cutoff is used and all edges vary in width and color. Defaults to NULL for graphs with less then 50 nodes and 0.3 to larger graphs.}
\item{details}{Logical indicating if minimum, maximum and cutoff score should be printed under the graph. Defaults to FALSE.}
}}

\section{Output arguments}{
\describe{
	\item{filetype}{A character containing the file type to save the output in. "R" and "x11" outputs in a new R window ("x11" is prefered for Rstudio), "pdf" creates a pdf file. "svg" creates a svg file (requires RSVGTipsDevice). 'jpg', 'tiff' and 'png' can also be used. If this is given any other string (e.g. filetype="") no device is opened. Defaults to 'R' if the current devide is the NULL-device or no new device if there already is an open device.}
	\item{filename}{Name of the file, if outputting in pdf}
	\item{width}{Width of the plot, in inches}
	\item{height}{Height of the plot, in inches}
}}

\section{Graphical arguments}{	
\describe{
\item{color}{A vector with a color for each element in the groups list, or a color for each node. Defaults to "white" without groups list and rainbow(length(groups)) with a groups list.}
	\item{legend}{Logical value indicating if a legend should be plotted}
	\item{legend.cex}{Scalar of the legend. defaults to 1}
	\item{GLratio}{Relative size of the graph compared to the layout. Defaults to 4}
	\item{vsize}{A value indicating the size of the nodes. Can also be a vector of length 2 (nodes are scaled to degree) or a size for each node.}
	\item{esize}{Size of the largest edge (or what it would be if there was an edge with weight maximum).}
	\item{labels}{If FALSE, no labels are plotted. If TRUE, order in adjacency matrix is used as labels. This can also be a vector with a label for each node. Defaults to TRUE. If a label contains an asterisk (e.g. "x1*") then the asterisk will be omitted and the label will be printed in symbol font (use this for Greek letters)}
	\item{edge.labels}{If FALSE, no edge labels are plotted. If TRUE, numerical edge weights are printed on the edges. This can also be a vector with a label for each edge. Defaults to FALSE. If a label contains an asterisk (e.g. "y1*") then the asterisk will be omitted and the label will be printed in symbol font (use this for Greek letters)}
	\item{edge.label.cex}{Either a single number or a number per edge used as a scalar of the edge label size. Defaults to 1.}
	\item{bg}{If this is TRUE, a background is plotted in which node colors cast a light of that color on a black background. Can also be a character containing the color of the background Defaults to FALSE}
	\item{bgcontrol}{The higher this is, the less light each node gives if bg=TRUE. Defaults to 6.}
	\item{bgres}{square root of the number of pixels used in bg=TRUE, defaults to 100.}
	\item{transparency}{In weighted graphs: logical indicating if the edges should fade to white or become more transparent (use this only if you use a background). In directed graphs this is a value between 0 and 1 indicating the level of transparancy.}
	\item{lcolor}{Character containing the color of the labels, defaults to "black"}
	\item{diag}{Should the diagonal also be plotted as edges? defaults to FALSE. Can also be "col" to plot diagonal values as vertex colors.}
	\item{loop}{If diag=T, this can be used to scale the size of the loop. defaults to 1.}
	\item{lty}{Line type, see 'par'}
	\item{borders}{Logical indicating if borders should be plotted, defaults to TRUE.}
	\item{shape}{A character containing the shape of the nodes. "circle", "square", "triangle" and "diamond" are supported. Can also be a vector with a shape for each node. Defaults to "circle".}
	\item{label.scale}{Logical indicating if labels should be scaled to fit the node. Defaults to TRUE.}
	\item{gray}{Logical, set to TRUE to plot the graph in grayscale colors}
	\item{tooltips}{A vector with tooltips for each node, only when filetype='svg'}
	\item{vTrans}{Transparency of the nodes, must be an integer between 0 and 255, 255 indicating no transparancy. Defaults to 255}
	\item{overlay}{Logical, should a Venn-diagram like overlay be plotted? If TRUE then for each group a x\% confidence region is plotted for the X and Y position, using \code{\link[ellipse]{ellipse}}}
	\item{overlaySize}{Specifies the size of the overlay ellipses. Corresponds to the confidence level (default is 0.5)}
}}

\section{Additional arguments for directed graphs}{
\describe{
	\item{directed}{Logical indicating if edges are directed or not. Can be TRUE or FALSE to indicate if all edges are directed, a logical vector (when using edgelists) or a logical matrix (when using adjacency matrix)}
	\item{curve}{A value indicating how strongly edges should be curved. Defaults to 0 (no curve) for each edge except for edges in directional graphs between nodes that have two edges in between them. CURVING LINES DRASTICALLY INCREASES FILE SIZE!}
	\item{arrows}{A logical indicating if arrows should be drawn, or a number indicating how much arrows should be drawn on each edge. If this is TRUE, a simple arrow is plotted, if this is a number, arrows are put in the middle of the edges.}
	\item{asize}{Scalar of the length of arrow heads, in inches. Defaults to 0.15.}
	\item{open}{Logical indicating if open (TRUE) or closed (FALSE) arrowheads should be drawn.}
	\item{bidirectional}{If this is TRUE, Then directional edges between nodes that have two edges between them are not curved.}
}}

\section{Special arguments for correlation matrices}{
\describe{
	\item{graph}{Type of graph to be made, for use with a correlation matrix as input. "association" will plot the matrix as is, "concentration" will first compute partial correlations between each pair of nodes (controlled for all other variables) and "factorial" will create a graph based on an exploratory factor analysis. Finally "sig" will transform all correlations in p-values (using the fdrtool package; Korbinian Strimmer, 2009) and force mode="sig". "sig2" will do the same but show p-values based on negative statistics in shades of orange}
	\item{mode}{This argument defines the mode used for coloring the edges. The default, "strength" assumes each edge weight indicates the strength of connection and makes positive edges green and negative edges red. If this is set to "sig" then the edge weights are assumed to be p-values and colored accordingly. This can also include negative values, which will be interpreted as p-values based on negative statitstics.}
	\item{graph}{A vector of max 4 elements indicating the alpha level cutoffs. Defaults to c(0.0001,0.001,0.01,0.05)}	
	\item{OmitInsig}{Logical indicating if edge weights with a p-value over the highest alpha level should be omitted. Defaults to FALSE, can be used with any mode}
	\item{sigScale}{The function used to scale the edges if mode="sig". Defaults to $function(x)0.8*(1-x)^(log(0.4/0.8,1-0.05))$}
	\item{bonf}{Logical indicating if a bonferonni correction should be applied if mode="sig". If so p-values are multiplied by the number of edges}
}}


\section{Additional arguments}{	
\describe{
	\item{scores}{This argument can be used to plot scores of an individual on the test. Should be a vector with the scores for each item. Defaults to NULL.}
	\item{scores.range}{Vector of length two indicating the range of the scores, of scores is not NULL.}
	\item{nNodes}{The number of nodes, only needs to be specified if the first argument is an edge-list and some nodes have no edges}
	\item{DoNotPlot}{Runs qgraph but does not plot. Useful for saving the output (i.e. layout) without plotting}
}}

\details{
The qgraph function has a lot of arguments. But most default values work very well, so often only a few arguments are needed. 
This version is not yet completed, so there could be bugs. For any remarks or questions, please contact the author.
}
\section{Creating graphs}{
This function creates a graph based on an adjacency matrix (recommended) or edgelist for the first argument ('adj'). The graph
will be directed or undirected and weighted or unweighted based on the input file (e.g. a symmetric adjacency
matrix with only ones and zeros will result in an undirected unweighted graph) but this can also be specified
with the 'weighted' and 'directed' arguments.

The main idea behind qgraph is to visualize a correlation matrix as a graph. If a correlation matrix is used as
input (recommended) a graph will be created with a node for each variable and edges between those nodes representing
the strength of the correlation between those variables (the correlation becomes the edge weight). Green edges represent positive correlations and red edges
represent negative correlations.

Correlation matrices can be graphed in three ways, indicated by the 'graph' argument. If graph="association" the edge weights correspond
directly to the correlations. If graph="concentration" then the edge weights become the partial correlations between each pair of nodes 
controlled for all other nodes. Finally if graph="factorial" an exploratory factor analysis is performed with \code{\link{factanal}} using
promax rotation and 'nfact' factors (defaults to half of the number of variables). An undirected graph is made in which each variable is connected
to another variable if they both load higher then the value of 'cut' on the same factor.

An edgelist can also be used. This is a matrix (not a list!) with two columns and a row for each edge. The first column indicates the number
of the start of the edge and the second column indicates the number of the end of the edge. Numbering must start with 1. If any
integer is missing between 1 and the highest specified number then that is regarded as a node with no edges (if there is
supposed to be more nodes with no edges after the highest number in the edgelist the 'nNodes' argument can be used).
To create a weighted graph edge weights can be added as a third column in the edgelist.}

\section{Interpreting graphs}{
The width of the edges is specified with the 'esize' arguments and corresponds to the 'lwd' argument in \code{\link{par}}.
Edges with an absolute value under the 'minimum' argument are omitted (useful to keep filesizes from inflating in very large 
graphs). If the 'cut' argument equals 0 (see next paragraph) An (hypothetical) edge with an absolute weight of the minimum value would be white and has width 1, Edges 
with absolute weights higher than the minimum will be wider and more colorful, and the edge with the strongest absolute
weight will be full red (negative) or green (positive) and have the width specified with 'esize'. It is possible to set this strongest edge by using
the 'maximum' argument. When 'maximum' is set to a value above any absolute weight in the graph that value is considered the strongest
edge (this must be done to compare different graphs).

In larger graphs the above edge settings can become very uninterpretable, and setting a cutoff score is advised.
This can be done with the 'cut' argument. With the 'cut' argument a cutoff value can be set which splits scaling
of color and width. Edges with absolute weights under the cutoff score will have width 1 and become more colorful
as they approach the cutoff score, and edges with absolute weights over the cutoff score will be full red or green
and become wider the stronger they are. 

The size of the nodes can be set with the 'vsize' argument. If this has two values then the nodes will be scaled according
to their degree (sum of the row and column of the node).}

\section{Specifying the layout}{
The placement of the nodes (i.e. the layout) is specified with the 'layout' argument. It can be manually specified
by entering a matrix for this argument. The matrix must have a row for each node and two columns indicating its X and
Y coordinate respectively. qgraph plots the nodes on a (-1:1)(-1:1) plane, and the given coordinates will be rescaled
to fit this plane unless 'rescale' is FALSE (not recommended). 

Another option to manually specify the layout is by entering a matrix with more then two columns. This matrix must then
consist of zeroes and a number (the order in the adjacency matrix) for each node indicating it's place. For example:

0 0 2 0 0

1 0 3 0 4

will place node 2 at the top in the center, node 1 at the bottom left corner, node 3 at the bottom in the center and node 4
at the bottom right corner.

It is recommended however that one of the integrated layouts is used. 'layout' can be given a character as argument to accomplish
that. layout="circle" will simply place all nodes in a circle and layout="groups" will place the nodes in separate circles per group
(see next section).

The circle and groups layouts and manual layouts are convenient to see how well the data conforms to a model, but to show how the data
clusters another layout is more appropriate. By specifying layout="spring" the Fruchterman-reingold algorithm (Fruchterman & Reingold, 1991), which has been ported
from the SNA package (Butts, 2010), can be used to create a force-directed layout. In principle, what this function does is that each node
(connected and unconnected) repulse each other, and connected nodes also attract each other. Then after a number of iterations (500 by default)
in which the maximum displacement of each node becomes smaller a layout is achieved in which the distance between nodes correspond very
well to the absolute edge weight between those nodes.

A solution to use this function for weighted graphs has been taken from the igraph package (Csardi G & Nepusz T, 2006) in which the same
function was ported from the SNA package. New in qgraph are the option to include constraints on the nodes by fixing
a coordinate for nodes or reducing the maximum allowed displacement per node. This can be done with the 'layout.par' argument.
For more information see \code{\link{qgraph.layout.fruchtermanreingold}}.

By default, 'layout' is set to "spring" for unweighted and directed graphs, "circle" to weighted undirected graphs (e.g. a correlation matrix)
without 'groups' specified (see next section) and "groups" to weighted undirected graphs when "groups" is specified.}

\section{Specifying a measurement model}{
A measurement model can be specified with the 'groups' argument. This must be a list in which each element is a vector
containing the numbers of nodes that belong together (numbers are taken from the order in the adjacency matrix). All numbers
must be included. If a groups list is specified the "groups" layout can be used to place these nodes together, the nodes
in each group will be given a color, and a legend can be plotted (by setting 'legend' to TRUE). The colors will be taken from the
'color' argument, or be generated with the \code{\link{rainbow}} function.}

\section{Output}{
By default qgraph will plot the graph in a new R window. However the graphs
are optimized to be plotted in a PDF file. To create a pdf file set the 'filetype' argument to "pdf". 'filename' can be used
to specify the filename and folder to output in. 'height' and 'width' can be used to specify the height and width of the image
in inches. By default a new R window is opened if the current device is the NULL-device, otherwise the current device is used (note that when doing this 'width' and 'height' still optimize the image for those
widths and heights, even though the output screen size isn't affected, this is especially important for directed graphs!).

The graphs can also be outputted in an SVG file using the RSVGTipsDevice package (Plate, 2009). An SVG image can be opened in most
browsers (firefox and chrome are recommended), and can be used to display tooltips. Each node can be given a tooltip
with the 'tooltips' argument. The function \code{\link{qgraph.svg}} can be used to make a battery of svg pictures with hyperlinks
to each other, working like a navigation menu.

IMPORTANT NOTE: RSVGTipsDevice is a 32-bit only package, so SVG functionality is not available in 64bit versions of R.}

\section{Additional information}{
By default, edges will be straight between two nodes unless there are two edges between two nodes. To overwrite this the 'bidirectional'
argument can be set to TRUE, which will turn two edges between two nodes into one bidirectional edge. 'bidirectional' can also
be a vector with TRUE or FALSE for each edge.

To specify the strength of the curve the argument 'curve' can be used (but only in directional graphs). 'curve' must be given a numerical value that
represent an offset from the middle of the straight edge through where the curved edge must be drawn. 0 indicates no curve, and any other value 
indicates a curve of that strength. A value of 0.3 is recommended for nice curves. This can be either one number or a vector with the curve of each edge.

Nodes and edges can be given labels with the 'labels' and the 'edge.labels' arguments. 'labels' can be set to FALSE to omit labels, TRUE (default) to set labels
equal to the node number (order in the adjacency matrix) or it can be a vector with the label for each node. Edge labels can also be set to FALSE to
be omitted (default). If 'edge.labels' is TRUE then the weight of each label is printed. Finally, 'edge.labels' can also be a vector with the label for each edge.
If a label (both for edges and nodes) contain an asterisk then the asterisk is omitted and that label is printed in the symbol font (useful to print Greek letters).

A final two things to try: the 'scores' argument can be given a vector with the scores of a person on each variable, which will then be shown using colors of the nodes,
 And the 'bg' argument can be used to change the background of the graph to another color, or use bg=TRUE for a special background (do set transparency=TRUE when using
 background colors other then white).}

\section{Debugging}{
If this function crashes for any reason, run:

dev.off()

To shut down the output device!} 
 
\value{
qgraph returns a 'qgraph' object that contains all the arguments used, with the exeption of the 'layout' argument which 
is set to the final layout used in the graph.
This can then be sent to a new qgraph function to use the same arguments in the new plot.

One of the values returned is 'layout.orig', the original (not rescaled) layout, which needs to be used when using constraint layouts.}


\references{
https://sites.google.com/site/qgraphproject

Carter T. Butts <buttsc@uci.edu> (2010). sna: Tools for Social Network Analysis. R package version 2.2-0. http://CRAN.R-project.org/package=sna

Csardi G, Nepusz T (2006). The igraph software package for complex network research, InterJournal, Complex Systems 1695. http://igraph.sf.net

Korbinian Strimmer (2009). fdrtool: Estimation and Control of (Local) False Discovery Rates. R package version 1.2.6. http://CRAN.R-project.org/package=fdrtool

Plate, T. <tplate@acm.org> and based on RSvgDevice by T Jake Luciani <jakeluciani@yahoo.com> (2009). RSVGTipsDevice: An R SVG graphics device with dynamic tips and hyperlinks. R package version 1.0-1.

Fruchterman, T. & Reingold, E. (1991). Graph drawing by force-directed placement. Software - Pract. Exp. 21, 11291164.

}
\author{
Sacha Epskamp<s.epskamp@uva.nl>
}

\seealso{
\code{\link{qgraph}}
\code{\link{qgraph.efa}}
\code{\link{qgraph.pca}}
\code{\link{qgraph.loadings}}
\code{\link{qgraph.sem}}
\code{\link{qgraph.lavaan}}
\code{\link{qgraph.cfa}}
\code{\link{qgraph.svg}}
\code{\link{qgraph.panel}}
}
\examples{
### BIG 5 DATASET ###
# Load big5 dataset:
data(big5)
data(big5groups)

# Correlations:
Q <- qgraph(cor(big5),minimum=0.25,cut=0.4,vsize=2,groups=big5groups,legend=TRUE,borders=FALSE)
title("Big 5 correlations",line=-2,cex.main=2)

# Same graph with spring layout:
Q <- qgraph(Q,layout="spring")
title("Big 5 correlations",line=-2,cex.main=2)

# Same graph with Venn diagram overlay:
qgraph(Q,overlay=TRUE)
title("Big 5 correlations",line=-2,cex.main=2)

# Significance graph (circular):
qgraph(Q,graph="sig",layout="circular")
title("Big 5 correlations (p-values)",line=-2,cex.main=2)

# Significance graph:
qgraph(Q,graph="sig")
title("Big 5 correlations (p-values)",line=-2,cex.main=2)

# Significance graph (distinguishing positive and negative statistics):
qgraph(Q,graph="sig2")
title("Big 5 correlations (p-values)",line=-2,cex.main=2)

# Grayscale graphs:
qgraph(Q,gray=TRUE,layout="circular")
title("Big 5 correlations",line=-2,cex.main=2)

qgraph(Q,graph="sig",gray=TRUE)
title("Big 5 correlations (p-values)",line=-2,cex.main=2)

# EFA:
big5efa <- factanal(big5,factors=5,rotation="promax",scores="regression")
qgraph(big5efa,groups=big5groups,layout="circle",rotation="promax",minimum=0.2,cut=0.4,vsize=c(1,15),borders=FALSE,asize=0.07,esize=4,vTrans=200,filetype="R",width=7,height=7)
title("Big 5 EFA",line=-2,cex.main=2)

# PCA:
library("psych")
big5pca <- principal(cor(big5),5,rotate="promax")
qgraph(big5pca,groups=big5groups,layout="circle",rotation="promax",minimum=0.2,cut=0.4,vsize=c(1,15),borders=FALSE,asize=0.07,esize=4,vTrans=200)
title("Big 5 PCA",line=-2,cex.main=2)

#### UNWEIGHTED DIRECTED GRAPHS ###
set.seed(1)
adj=matrix(sample(0:1,10^2,TRUE,prob=c(0.8,0.2)),nrow=10,ncol=10)
qgraph(adj)
title("Unweighted and directed graphs",line=-2,cex.main=2)

# Save plot to nonsquare pdf file:
\dontrun{
qgraph(adj,filetype='pdf',height=5,width=10)
}

#### EXAMPLES FOR EDGES UNDER DIFFERENT ARGUMENTS ###
# Create edgelist:
dat.3 <- matrix(c(1:15*2-1,1:15*2),,2)
dat.3 <- cbind(dat.3,round(seq(-0.7,0.7,length=15),1))

# Create grid layout:
L.3 <- matrix(1:30,nrow=2)

# Different esize:
qgraph(dat.3,layout=L.3,directed=FALSE,edge.labels=TRUE,esize=14)

# Different esize, strongest edges omitted (note how 0.4 edge is now 
# just as wide as 0.7 edge in previous graph):
qgraph(dat.3[-c(1:3,13:15),],layout=L.3,nNodes=30,directed=FALSE,
	edge.labels=TRUE,esize=14)

# Different esize, with maximum:
qgraph(dat.3,layout=L.3,directed=FALSE,edge.labels=TRUE,esize=14,maximum=1)
title("maximum=1",line=-2,cex.main=2)

qgraph(dat.3[-c(1:3,13:15),],layout=L.3,nNodes=30,directed=FALSE,edge.labels=TRUE,
	esize=14,maximum=1)
title("maximum=1",line=-2,cex.main=2)

# Different minimum
qgraph(dat.3,layout=L.3,directed=FALSE,edge.labels=TRUE,esize=14,minimum=0.1)
title("minimum=0.1",line=-2,cex.main=2)

# With cutoff score:
qgraph(dat.3,layout=L.3,directed=FALSE,edge.labels=TRUE,esize=14,cut=0.4)
title("cut=0.4",line=-2,cex.main=2)

# With details:
qgraph(dat.3,layout=L.3,directed=FALSE,edge.labels=TRUE,esize=14,minimum=0.1,maximum=1,
	cut=0.4,details=TRUE)
title("details=TRUE",line=-2,cex.main=2)


### STRUCTURAL EQUATION MODELLING ###
\dontrun{
library('sem')

### This example is taken from the examples of the sem function. 
### Only names were changed to better suit the path diagram.

# ----------------------- Thurstone data ---------------------------------------
#  Second-order confirmatory factor analysis, from the SAS manual for PROC CALIS

R.thur <- read.moments(diag=FALSE, names=c('Sen','Voc',
        'SC','FL','4LW','Suf',
        'LS','Ped', 'LG'))
    .828                                              
    .776   .779                                        
    .439   .493    .46                                 
    .432   .464    .425   .674                           
    .447   .489    .443   .59    .541                    
    .447   .432    .401   .381    .402   .288              
    .541   .537    .534   .35    .367   .32   .555        
    .38   .358    .359   .424    .446   .325   .598   .452  
            
model.thur <- specify.model()
    F1 -> Sen,               *l11, NA
    F1 -> Voc,               *l21, NA
    F1 -> SC,                *l31, NA
    F2 -> FL,                *l41, NA
    F2 -> 4LW,               *l52, NA
    F2 -> Suf,               *l62, NA
    F3 -> LS,                *l73, NA
    F3 -> Ped,               *l83, NA
    F3 -> LG,                *l93, NA
    F4 -> F1,                *g1,  NA
    F4 -> F2,                *g2,  NA
    F4 -> F3,                *g3,  NA 
    Sen <-> Sen,             q*1,   NA
    Voc<-> Voc,              q*2,   NA
    SC <-> SC,               q*3,   NA
    FL <-> FL,               q*4,   NA
    4LW <-> 4LW,             q*5,   NA
    Suf<-> Suf,              q*6,   NA
    LS <-> LS,               q*7,   NA
    Ped<-> Ped,              q*8,   NA
    LG <-> LG,               q*9,   NA
    F1 <-> F1,               NA,     1
    F2 <-> F2,               NA,     1
    F3 <-> F3,               NA,     1
    F4 <-> F4,               NA,     1
}

\dontshow{
require('sem')

### This example is taken from the examples of the sem function. 
### Only names were changed to better suit the path diagram.

# ----------------------- Thurstone data ---------------------------------------
#  Second-order confirmatory factor analysis, from the SAS manual for PROC CALIS

R.thur <- structure(c(1, 0.828, 0.776, 0.439, 0.432, 0.447, 0.447, 0.541, 
0.38, 0, 1, 0.779, 0.493, 0.464, 0.489, 0.432, 0.537, 0.358, 
0, 0, 1, 0.46, 0.425, 0.443, 0.401, 0.534, 0.359, 0, 0, 0, 1, 
0.674, 0.59, 0.381, 0.35, 0.424, 0, 0, 0, 0, 1, 0.541, 0.402, 
0.367, 0.446, 0, 0, 0, 0, 0, 1, 0.288, 0.32, 0.325, 0, 0, 0, 
0, 0, 0, 1, 0.555, 0.598, 0, 0, 0, 0, 0, 0, 0, 1, 0.452, 0, 0, 
0, 0, 0, 0, 0, 0, 1), .Dim = c(9L, 9L), .Dimnames = list(c("Sen", 
"Voc", "SC", "FL", "4LW", "Suf", "LS", "Ped", "LG"), c("Sen", 
"Voc", "SC", "FL", "4LW", "Suf", "LS", "Ped", "LG")))
            
model.thur <- structure(c("F1 -> Sen", "F1 -> Voc", "F1 -> SC", "F2 -> FL", 
"F2 -> 4LW", "F2 -> Suf", "F3 -> LS", "F3 -> Ped", "F3 -> LG", 
"F4 -> F1", "F4 -> F2", "F4 -> F3", "Sen <-> Sen", "Voc<-> Voc", 
"SC <-> SC", "FL <-> FL", "4LW <-> 4LW", "Suf<-> Suf", "LS <-> LS", 
"Ped<-> Ped", "LG <-> LG", "F1 <-> F1", "F2 <-> F2", "F3 <-> F3", 
"F4 <-> F4", "*l11", "*l21", "*l31", "*l41", "*l52", "*l62", 
"*l73", "*l83", "*l93", "*g1", "*g2", "*g3", "q*1", "q*2", "q*3", 
"q*4", "q*5", "q*6", "q*7", "q*8", "q*9", NA, NA, NA, NA, NA, 
NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, 
NA, NA, NA, NA, "1", "1", "1", "1"), .Dim = c(25L, 3L), class = "mod")
}

# Path diagram of the model:
qgraph(model.thur)
qgraph(model.thur,layout="tree",manifest=c('Sen','Voc','SC','FL','4LW','Suf','LS','Ped','LG'))

# Estimate and plot parameters:
sem.thur <- sem(model.thur, R.thur, 213)
qgraph(sem.thur,layout="tree",curve=0.4)

\dontrun{
### pcalg support ###
# Example from pcalg vignette:
library("pcalg")
data(gmI)
suffStat <- list(C = cor(gmI$x), n = nrow(gmI$x))
pc.fit <- pc(suffStat, indepTest=gaussCItest,
p = ncol(gmI$x), alpha = 0.01)

qgraph(pc.fit)
}
}


% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ Graphs }
\keyword{ qgraph }
\keyword{ Correlations}
